{===============================================================================
  Unit QRCustomPreview

  This unit contains the TQRPreviewForm form that can be used for custom
  QuickReport previews. You need to install the TQRPreviewToolBar component
  before using this. To use it just call the ShowPreview method on the
  OnPreview event of your TQuickReport.
  Tested with Delphi 3 and QuickReport 2.

  Author:      Craig Manley
  Created:     24 November 1999
  Last update: 26 November 1999
===============================================================================}

unit QRCustomPreview;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms,
  StdCtrls, ExtCtrls, Spin, qrprntr,quickrpt,qr2const,Buttons,
  ToolWin, Dialogs, ComCtrls, LabelSpinEdit, QRPreviewToolBar, qrptcnst;

type
  TWMWindowPosChanging = record
    Msg: Cardinal;
    Unused: Longint;
    lpwp: PWindowPos;
    Result: Longint;
  end;


  TQRPreviewForm = class(TForm)
    QRPreview: TQRPreview;
    SaveDialogExport: TSaveDialog;
    pnlTop: TPanel;
    QRPreviewToolBar1: TQRPreviewToolBar;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure QRPreviewPageAvailable(Sender: TObject; PageNum: Integer);
    procedure CancelReport;
    procedure QRPreviewToolBar1Save(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    FStrFiles,
    FStrFailLoadFile,
    FStrFinished,
    FStrReady,
    FStrBusy,
    FStrPage,
    FStrPages,
    FStrCancelReport,
    FStrGeneratingReport,
    FStrExportReportAs,
    FStrUntitled: string;
    FTitle: string;
    procedure GoToFirst;
    procedure GoToPrevious;
    procedure GoToNext;
    procedure GoToLast;
    procedure SetLanguage(Language: TQRLanguage);
    procedure WMWindowPosChangingProc(var Msg: TWMWindowPosChanging); message WM_WINDOWPOSCHANGING;
  public
    procedure ShowPreview(QuickRep: TQuickRep; QRPrinter: TQRPrinter);
  published
  end;

var
  QRPreviewForm: TQRPreviewForm;

implementation

{$R *.DFM}
{----------------------------- Private methods --------------------------------}
procedure TQRPreviewForm.GoToFirst;
begin
  QRPreview.PageNumber := 1;
  QRPreviewToolBar1.ResetPageNumber;
end;

procedure TQRPreviewForm.GoToPrevious;
begin
  if QRPreview.PageNumber > 1 then
  begin
    QRPreview.PageNumber := QRPreview.PageNumber - 1;
    QRPreviewToolBar1.ResetPageNumber;
  end;
end;

procedure TQRPreviewForm.GoToNext;
begin
  if QRPreview.PageNumber < QRPreview.QRPrinter.PageCount then
  begin
    QRPreview.PageNumber := QRPreview.PageNumber + 1;
    QRPreviewToolBar1.ResetPageNumber;
  end;
end;

procedure TQRPreviewForm.GoToLast;
begin
  QRPreview.PageNumber := QRPreview.QRPrinter.PageCount;
  QRPreviewToolBar1.ResetPageNumber;
end;

procedure TQRPreviewForm.SetLanguage(Language: TQRLanguage);
begin
  case Language of
    qrlgDutch:
    begin
      FStrPage             := SNLPage;
      FStrPages            := SNLPages;
      FStrFiles            := SNLFiles;
      FStrFailLoadFile     := SNLFailLoadFile;
      FStrFinished         := SNLFinished;
      FStrReady            := SNLReady;
      FStrBusy             := SNLBusy;
      FStrCancelReport     := SNLCancelReport;
      FStrGeneratingReport := SNLGeneratingReport;
      FStrExportReportAs   := SNLExportReportAs;
      FStrUntitled         := SNLUntitled;
    end;
    qrlgEnglish:
    begin
      FStrPage             := SENPage;
      FStrPages            := SENPages;
      FStrFiles            := SENFiles;
      FStrFailLoadFile     := SENFailLoadFile;
      FStrFinished         := SENFinished;
      FStrReady            := SENReady;
      FStrBusy             := SENBusy;
      FStrCancelReport     := SENCancelReport;
      FStrGeneratingReport := SENGeneratingReport;
      FStrExportReportAs   := SENExportReportAs;
      FStrUntitled         := SENUntitled;
    end;
  end;
end;

procedure TQRPreviewForm.WMWindowPosChangingProc(var Msg: TWMWindowPosChanging);
begin
  if self.Visible then
  begin
    Msg.lpwp^.flags := Msg.lpwp^.flags or SWP_NOMOVE or SWP_NOSIZE;
    Msg.Result := 0;
  end;
  inherited;
end;

{----------------------------- Public methods ---------------------------------}
procedure TQRPreviewForm.ShowPreview(QuickRep: TQuickRep; QRPrinter: TQRPrinter);
begin
  if self.Showing then
    Close;
  QRPreviewToolBar1.QuickRep := QuickRep;
  QRPreview.QRPrinter := QRPrinter;
  FTitle := QRPreview.QRPrinter.Title;
  if FTitle = '' then
    FTitle := FStrUntitled;
  self.Caption := FStrGeneratingReport;
  Show;
  if self.WindowState = wsMinimized then
    self.WindowState := wsMaximized;
end;

procedure TQRPreviewForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  QRPreview.QRPrinter := nil;
end;

procedure TQRPreviewForm.QRPreviewPageAvailable(Sender: TObject; PageNum: Integer);
begin
  { Display the number of rendered pages }
  if PageNum = 1 then
  begin
    QRPreviewToolBar1.Reset;
    Caption := FTitle + ' - 1 ' + FStrPage;
  end
  else
    Caption := FTitle + ' - ' + IntToStr(PageNum) + ' ' + FStrPages;
  { Here's one way to show the current status of the report }
  case QRPreview.QRPrinter.Status of
    mpReady:    Caption := Caption + ' (' + FStrReady + ')';
    mpBusy:     Caption := Caption + ' (' + FStrBusy + ')';
    mpFinished: Caption := Caption + ' (' + FStrFinished + ')';
  end;
end;

procedure TQRPreviewForm.CancelReport;
begin
  if QRPreview.QRPrinter.Status = mpBusy then
    if MessageDlg(FStrCancelReport, mtConfirmation, mbOKCancel,0) = mrOK then
    begin
      QRPrinter.Cancel;
      QRPreview.QRPrinter := nil;
    end;
end;

procedure TQRPreviewForm.QRPreviewToolBar1Save(Sender: TObject);
var
  aExportFilter : TQRExportFilter;
begin
  aExportFilter := nil;
  with TSaveDialog.Create(Application) do
  try
    Title := FStrExportReportAs;
    Filter := QRExportFilterLibrary.SaveDialogFilterString;
{    DefaultExt := cQRPDefaultExt;}
    if Execute then
    begin
      if FilterIndex = 1 then
      begin
        if ExtractFileExt(Filename) = '' then
          Filename := Filename + '.qrp';
        QRPreview.QRPrinter.Save(Filename);
      end
      else
      begin
        try
          aExportFilter := TQRExportFilterLibraryEntry(
            QRExportFilterLibrary.Filters[FilterIndex - 2]).ExportFilterClass.Create(Filename);
          QRPreview.QRPrinter.ExportToFilter(aExportFilter);
        finally
          aExportFilter.Free
        end
      end
    end;
  finally
    Free;
  end;
end;

procedure TQRPreviewForm.FormCreate(Sender: TObject);
begin
  SetLanguage(qrlgDutch);
end;

procedure TQRPreviewForm.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (QRPreview.QRPrinter <> nil) and (QRPreview.QRPrinter.PageCount > 0) then
  begin
    case Key of
      VK_HOME: GotoFirst;
      VK_END: GotoLast;
      VK_NEXT: GotoNext;
      VK_PRIOR: GotoPrevious;
      VK_UP:
      begin
        GotoNext;
        Key := 0;
      end;
      VK_DOWN:
      begin
        GotoPrevious;
        Key := 0;
      end;
    end; { case }
  end;
end;

procedure TQRPreviewForm.FormShow(Sender: TObject);
begin
  QRPreviewToolBar1.Reset;
end;

procedure TQRPreviewForm.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (QRPreview.QRPrinter <> nil) and (QRPreview.QRPrinter.PageCount > 0) then
  begin
    case Key of
      VK_UP: Key := 0;
      VK_DOWN: Key := 0;
    end;
  end;
end;

end.
